@@ -4,7 +4,7 @@ AnyEvent::SOCKS::Client - AnyEvent-based SOCKS client!
 
 =head1 VERSION
 
-Version 0.04
+Version 0.05
 
 =cut
 
@@ -14,24 +14,24 @@ Constructs function which behave like AnyEvent::Socket::tcp_connect
 
     use AnyEvent::SOCKS::Client qw/tcp_connect_via/;
 
-	$AnyEvent::SOCKS::Client::TIMEOUT = 30; 
-	# used only if prepare_cb NOT passed to proxied function
-	# e.g. AE::HTTP on_prepare callback is not present
+    $AnyEvent::SOCKS::Client::TIMEOUT = 30;
+    # used only if prepare_cb NOT passed to proxied function
+    # e.g. AE::HTTP on_prepare callback is not present
 
     my @chain = qw/socks5://user:pass@192.0.2.100:1080 socks5://198.51.100.200:9080/;
     tcp_connect_via( @chain )->( 'example.com', 80, sub{
         my ($fh) = @_ or die "Connect failed $!";
-		...
+        ...
     }); 
 
 SOCKS client for AnyEvent::HTTP
 
-   http_get "http://example.com/foo", 
-      tcp_connect => tcp_connect_via('socks5://198.51.100.200:9080'), 
-      sub{ 
-         my( $data, $header) = @_ ;
-         ...
-      };
+    http_get "http://example.com/foo",
+        tcp_connect => tcp_connect_via('socks5://198.51.100.200:9080'),
+        sub{
+            my( $data, $header) = @_ ;
+            ...
+        };
 
 =head1 SECURITY
 
@@ -62,8 +62,10 @@ use AnyEvent::Socket qw/tcp_connect parse_ipv4 format_ipv4 parse_ipv6 format_ipv
 use AnyEvent::Handle ;
 use AnyEvent::Log ;
 
+use Scalar::Util qw/weaken/;
+
 require Exporter;
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 our @ISA = qw/Exporter/;
 our @EXPORT_OK = qw/tcp_connect_via/;
 
@@ -95,43 +97,61 @@ sub _parse_uri{
 }
 # returns tcp_connect compatible function
 sub tcp_connect_via{
-	my(@chain) = @_ ; 
+	my(@chain) = @_ ;
+
+	unless( @chain ){
+		AE::log "error" => "No socks were given, abort"; 
+		return sub{ $_[2]->() };
+	}
+	my @parsed;
+	for(@chain){
+		if( my $p = _parse_uri($_) ){
+			push @parsed, $p; next;
+		}
+		AE::log "error" => "Invalid socks uri: $_";
+		return sub{ $_[2]->() };
+	}
+
 	return sub{
-		my $con = bless { chain => [ map _parse_uri($_), @chain ] }, __PACKAGE__ ;
-		$con->connect( @_ ) ;
-		guard { undef };
+		my( $dst_host, $dst_port, $c_cb, $pre_cb ) = @_ ;
+		my $con = bless {
+			chain => \@parsed,
+			dst_host => $dst_host,
+			dst_port => $dst_port,
+			c_cb => $c_cb,
+			pre_cb => $pre_cb,
+		}, __PACKAGE__ ;
+		$con->connect;
+
+		if( defined wantarray and not wantarray ){ # scalar
+			weaken( $con );
+			return guard{
+				AE::log "debug" => "Guard triggered" ;
+				$con->DESTROY if( ref $con eq __PACKAGE__ );
+			};
+		}
+		undef;
 	};
 }
 
 sub connect{
-	my( $self, $dst_host, $dst_port, $c_cb, $pre_cb ) = @_ ;
-	unless( @{ $self->{chain} } ){
-		AE::log "error" => "No socks were given, abort"; 
-		return $c_cb->();	
-	}
-	if( $self->{c_cb}){
-		AE::log "error" => "It's one-off object, create another instance.."; 
-		return $c_cb->();
-	}
-	$self->{dst_host} = $dst_host; 
-	$self->{dst_port} = $dst_port;
-	$self->{c_cb} = $c_cb ;
-
+	my( $self ) = @_ ;
 	# tcp connect to first socks
 	my $that = $self->{chain}->[0] ;
-	return tcp_connect $that->{host}, $that->{port}, sub{
+	$self->{_guard} = tcp_connect $that->{host}, $that->{port}, sub{
 		my $fh = shift ;
 		unless($fh){
 			AE::log "error" => "$that->{host}:$that->{port} connect failed: $!";
-			return $c_cb->();
+			return;
 		}
+
 		$self->{hd} = new AnyEvent::Handle(
 			fh => $fh,
 			on_error => sub{
 				my ($hd, $fatal, $msg) = @_;
-				AE::log "error" => ( $fatal ? "Fatal " : "" ) . $msg ; 
+				AE::log "error" => ( $fatal ? "Fatal " : "" ) . $msg ;
 				$hd->destroy unless( $hd->destroyed );
-				$c_cb->();
+				return;
 			}
 		);
 		if($that->{v} =~ /4a?/){
@@ -139,7 +159,7 @@ sub connect{
 			return;
 		}
 		$self->handshake;
-	}, $pre_cb || sub{ $TIMEOUT };
+	}, $self->{pre_cb} || sub{ $TIMEOUT };
 }
 
 sub connect_socks4{
@@ -159,7 +179,7 @@ sub connect_socks4{
 		AE::log "error" => "SOCKS4/4a doesn't support IPv6 addresses: $host given";
 		return;
 	}
-
+	AE::log "debug" => "SOCKS4 connect to $host:$port";
 	$self->{hd}->push_write( $ip4 
 		? pack('CCnA4A2', 4, CMD_CONNECT, $port, $ip4, "X\0" )
 		: pack('CCnCCCCA*', 4, CMD_CONNECT, $port, 0,0,0,7 , "X\0$host\0" )
@@ -300,15 +320,19 @@ sub socks_connect_done{
 	}
 
 	AE::log "debug" => "Giving up fh and returning to void...";
-	$self->{c_cb}->( $self->{hd}->fh ) and undef $self->{c_cb};
+	my( $fh, $c_cb ) = ( $self->{hd}->fh, delete $self->{c_cb} );
+	$self->DESTROY;
+	$c_cb->( $fh );
 }
 
 sub DESTROY {
 	my $self = shift ;
 	AE::log "debug" => "Kitten saver called";
+	undef $self->{_guard};
 	$self->{hd}->destroy	if( $self->{hd} and not $self->{hd}->destroyed );
 	$self->{c_cb}->()		if( $self->{c_cb} );
 	undef %$self;
+	bless $self, __PACKAGE__ . '::destroyed';
 }
 
 
